# -*- coding: utf-8 -*-

import base64, json

from django.core.paginator import Paginator
from django.db.models import Q
from django.utils.translation import ugettext as _
from django.conf import settings
from django.contrib import admin
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from jmb.showcase.models import Item
from jmb.showcase.cms.models import (
    ItemSettings, ShowcaseFilterSettings, ItemCategory, ShowcaseSelectFilterSettings
)
from jmb.showcase.cms.forms import NewsCategoriesSelectForm


###
# INLINES
###
class ItemCategoryInline(admin.TabularInline):
    model = ItemCategory
    extra = 0
    fields = ('ordering', 'category')


###################################
## CMS PLUGINS                    #
###################################

class ItemPlugin(CMSPluginBase):
    model = ItemSettings
    name = _("Showcase Item List")
    module = _("Showcase")
    fieldsets = (
        (_('general info'), {'fields': (
            'name',
            'slugs_included', 'tales_included', 'slugs_excluded',
            'tales_excluded', 'ordering', 'objects_per_page', 'template_name'
        )}),
    )
    render_template = "bootstrap3/showcase/cms_plugins/empty.html"
    admin_preview = False
    template_name_choices = settings.SHOWCASE_CMSPLUGIN_TEMPLATES

    def get_form(self, request, obj=None, **kwargs):
        form = super(ItemPlugin, self).get_form(request, obj, **kwargs)
        form.base_fields['template_name'].choices = self.template_name_choices
        return form

    def render(self, context, instance, placeholder):
        page = 1
        self.render_template = instance.template_name
        objects = Item.objects.filter(status=1)

        ordering = instance.ordering.lower().replace(" ", "").split(",")

        if instance.slugs_included:
            slugs_inclusions = instance.slugs_included.replace(" ", "").split(",")
            objects = objects.filter(slug__in=slugs_inclusions)
        if instance.tales_included:
            tales_inclusions = instance.tales_included.replace(" ", "").split(",")
            objects = objects.filter(tale__in=tales_inclusions)
        if instance.slugs_excluded:
            slugs_exclusions = instance.slugs_excluded.lower().replace(" ", "").split(",")
            objects = objects.exclude(slug__in=slugs_exclusions)
        if instance.tales_excluded:
            tales_exclusions = instance.tales_excluded.lower().replace(" ", "").split(",")
            objects = objects.exclude(tale__in=tales_exclusions)

        objects = objects.order_by(*ordering)

        request = context['request']
        if request.GET:
            if 'page' in request.GET:
                page = request.GET['page']

        paginator = Paginator(objects, instance.objects_per_page)
        objects = paginator.page(page)
        context.update({
            'objects': objects,
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class ItemByCategoriesPlugin(ItemPlugin):
    name = _("Showcase Item By Categories List")
    fieldsets = (
        (_('general info'), {'fields': (
            'name', 'ordering', 'show_pagination', 'objects_per_page', 'adjacent_pages', 'template_name'
        )}),
    )
    template_name_choices = settings.SHOWCASE_CMSPLUGIN_BY_CATEGORY_TEMPLATES
    inlines = (ItemCategoryInline,)
    render_template = "bootstrap3/showcase/cms_plugins/empty.html"
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        url = request.build_absolute_uri().split("?")[0]
        self.render_template = instance.template_name
        categories = [category.category for category in instance.categories.all()]
        ordering = instance.ordering.lower().replace(" ", "").split(",")

        objects = Item.objects.filter(status=1).filter(categories__in=categories).order_by(*ordering).distinct()
        if instance.language:
            objects = objects.filter(Q(language=instance.language) | Q(language__isnull=True))
        # Pagination
        page = 1
        instance_filter_key = "scf_{0}".format(instance.id)
        if instance_filter_key in request.GET:
            try:
                query_string = json.loads(base64.b64decode(request.GET[instance_filter_key]))
                if 'page' in query_string:
                    page = int(query_string['page'])
                if 'categories' in query_string:
                    objects = objects.filter(categories__id__in=query_string['categories'])
            except (TypeError, ValueError):
                pass
        paginator = Paginator(objects, instance.objects_per_page)
        objects = paginator.page(page)
        adjacent_pages = instance.adjacent_pages
        page_numbers = [n for n in range(page-adjacent_pages, page+adjacent_pages+1) if 0 < n <= paginator.num_pages]
        show_first = True if 1 not in page_numbers else False
        show_last = True if paginator.num_pages not in page_numbers else False
        context.update({
            'categories': categories,
            'page': page,
            'prev_page': page-1,
            'next_page': page+1,
            'show_first': show_first,
            'show_last': show_last,
            'page_numbers': page_numbers,
            'paginator': paginator,
            'objects': objects,
            'instance': instance,
            'placeholder': placeholder,
            'url': url,
        })
        return context


class ShowcaseFilterPlugin(CMSPluginBase):
    model = ShowcaseFilterSettings
    name = _("Showcase Filter")
    module = _("Showcase")
    render_template = "bootstrap3/showcase/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        self.render_template = instance.template_name
        categories = [
            category.category for category in
            instance.item_plugin_setting.categories.all().order_by('ordering')
        ]
        selected_categories = None
        instance_filter_key = "scf_{0}".format(instance.item_plugin_setting.id)
        if instance_filter_key in request.GET:
            try:
                query_string = json.loads(base64.b64decode(request.GET[instance_filter_key]))
                if 'categories' in query_string:
                    selected_categories = [int(id) for id in query_string['categories']]
            except (TypeError, ValueError):
                pass
        context.update({
            'instance': instance,
            'setting': instance.item_plugin_setting,
            'placeholder': placeholder,
            'categories': categories,
            'selected_categories': selected_categories,
        })
        return context


class ShowcaseCategoriesSelectFilterPlugin(CMSPluginBase):
    model = ShowcaseSelectFilterSettings
    name = _("Showcase categories select filter")
    module = _("Showcase")
    render_template = "bootstrap3/showcase/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        selected_categories = []
        self.render_template = instance.template_name
        instance_filter_key = "scf_{0}".format(instance.item_plugin_setting.id)
        if instance_filter_key in request.GET:
            try:
                query_string = json.loads(base64.b64decode(request.GET[instance_filter_key]))
                if 'categories' in query_string:
                    selected_categories = [int(id) for id in query_string['categories']]
            except (TypeError, ValueError):
                pass
        form = NewsCategoriesSelectForm(
            setting=instance.item_plugin_setting, selected_categories=selected_categories
        )
        context.update({
            'instance': instance,
            'setting': instance.item_plugin_setting,
            'placeholder': placeholder,
            'form': form,
        })
        return context


plugin_pool.register_plugin(ItemPlugin)
plugin_pool.register_plugin(ItemByCategoriesPlugin)
plugin_pool.register_plugin(ShowcaseFilterPlugin)
plugin_pool.register_plugin(ShowcaseCategoriesSelectFilterPlugin)